@@ -11,11 +11,11 @@ venv=${1:-"functional"}
 function generate_testr_results {
     # Give job user rights to access tox logs
     sudo -H -u $owner chmod o+rw .
-    sudo -H -u $owner chmod o+rw -R .testrepository
-    if [ -f ".testrepository/0" ] ; then
-        .tox/$venv/bin/subunit-1to2 < .testrepository/0 > ./testrepository.subunit
-        $SCRIPTS_DIR/subunit2html ./testrepository.subunit testr_results.html
-        gzip -9 ./testrepository.subunit
+    sudo -H -u $owner chmod o+rw -R .stestr
+    if [ -f ".stestr/0" ] ; then
+        .tox/$venv/bin/subunit-1to2 < .stestr/0 > ./stestr.subunit
+        $SCRIPTS_DIR/subunit2html ./stestr.subunit testr_results.html
+        gzip -9 ./stestr.subunit
         gzip -9 ./testr_results.html
         sudo mv ./*.gz /opt/stack/logs/
     fi
@@ -62,10 +62,15 @@ if [[ "$venv" == *functional* ]] || [[ "$venv" == *fullstack* ]]; then
     # Run tests
     echo "Running neutron $venv test suite"
     set +e
-    sudo -H -u $owner $sudo_env tox -e $venv
+    sudo -E -H -u $owner $sudo_env tox -ve $venv
     testr_exit_code=$?
     set -e
 
+    # move and zip tox logs into log directory
+    sudo mv $PROJECT_DIR/.tox/$venv/log /opt/stack/logs/tox
+    sudo -H -u $owner chmod o+rw -R /opt/stack/logs/tox/
+    gzip -9 /opt/stack/logs/tox/*.log
+
     # Collect and parse results
     generate_testr_results
     generate_log_index
