@@ -1,6 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-# encoding: utf-8
-
 # Copyright 2017 Orange
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,7 +14,6 @@
 
 import logging as python_logging
 import time
-import traceback
 import uuid
 
 from oslo_config import cfg
@@ -91,6 +87,7 @@ class AttachController(VPNManagerController):
                         ip_address as a singleton (/32)
     'linuxbr': Name of a linux bridge to which the linuxif is already
              plugged-in (optional)
+    'vni': VXLAN VNI to use (optional)
     'local_pref': BGP LOCAL_PREF for the route to this vif (optional)
     'direction': 'to-port' | 'from-port' | 'both'
         # specify local port traffic direction in VPN instance
@@ -218,10 +215,9 @@ class AttachController(VPNManagerController):
         except exc.APIException as e:
             LOG.warning('attach_localport: API error: %s', e)
             pecan.abort(400, "API error: %s" % e)
-        except Exception as e:
-            LOG.error('attach_localport: An error occurred during local port'
-                      ' plug to VPN: %s', e)
-            LOG.info(traceback.format_exc())
+        except Exception:
+            LOG.exception('attach_localport: An error occurred during local '
+                          'port plug to VPN')
             pecan.abort(500, 'An error occurred during local port plug to VPN')
 
 
@@ -250,10 +246,9 @@ class DetachController(VPNManagerController):
         except exc.APIException as e:
             LOG.warning('detach_localport: API error: %s', e)
             pecan.abort(400, "API error: %s" % e)
-        except Exception as e:
-            LOG.error('detach_localport: An error occurred during local port'
-                      ' unplug from VPN: %s', e)
-            LOG.info(traceback.format_exc())
+        except Exception:
+            LOG.exception('detach_localport: An error occurred during local '
+                          'port unplug from VPN')
             pecan.abort(500, 'An error occurred during local port unplug from '
                         'VPN')
 
@@ -298,9 +293,8 @@ class LookingGlassController(VPNManagerController,
         except lg.NoSuchLookingGlassObject as e:
             LOG.info('looking_glass: %s', repr(e))
             pecan.abort(404, repr(e))
-        except Exception as e:
-            LOG.error('looking_glass: An error occurred: %s', e)
-            LOG.error(traceback.format_exc())
+        except Exception:
+            LOG.exception('looking_glass: an error occurred')
             pecan.abort(500, 'Server error')
 
     @when(index, method='DELETE')
@@ -313,11 +307,11 @@ class LookingGlassController(VPNManagerController,
 
     def get_lg_map(self):
         return {
-            "summary":  (lg.SUBITEM, self.get_lg_summary),
-            "config":   (lg.SUBTREE, self.get_lg_config),
-            "bgp":      (lg.DELEGATE, self.manager.bgp_manager),
-            "vpns":     (lg.DELEGATE, self.manager),
-            "logs":     (lg.SUBTREE, self.get_logs)
+            "summary": (lg.SUBITEM, self.get_lg_summary),
+            "config": (lg.SUBTREE, self.get_lg_config),
+            "bgp": (lg.DELEGATE, self.manager.bgp_manager),
+            "vpns": (lg.DELEGATE, self.manager),
+            "logs": (lg.SUBTREE, self.get_logs)
         }
 
     def get_lg_config(self, path_prefix):
@@ -331,18 +325,13 @@ class LookingGlassController(VPNManagerController,
         return {
             "BGP_established_peers":
                 self.manager.bgp_manager.get_established_peers_count(),
-            "local_routes_count":
-                self.manager.bgp_manager.rtm.
-                get_local_routes_count(),
-            "received_routes_count":
-                self.manager.bgp_manager.rtm.
-                get_received_routes_count(),
+            "route_counts": self.manager.bgp_manager.get_lg_route_counts(),
             "vpn_instances_count": self.manager.get_vpn_instances_count(),
             "warnings_and_errors": len(self.catchall_lg_log_handler),
             "start_time": time.strftime("%Y-%m-%d %H:%M:%S",
                                         time.localtime(self.start_time)),
-            "version":  (pbr.version.VersionInfo('networking-bagpipe')
-                         .release_string())
+            "version": (pbr.version.VersionInfo('networking-bagpipe')
+                        .release_string())
         }
 
     def get_logs(self, path_prefix):
