@@ -20,12 +20,16 @@ import logging
 import os
 import re
 import sys
+import urllib.error
+import urllib.request
+
 
 import netaddr
 import optparse
+from oslo_config import cfg
 from oslo_serialization import jsonutils
-import urllib2
 
+from networking_bagpipe.bagpipe_bgp.api import config as api_config
 from networking_bagpipe.bagpipe_bgp.common import net_utils
 from networking_bagpipe.bagpipe_bgp.common import run_command
 from networking_bagpipe.bagpipe_bgp import constants as const
@@ -123,6 +127,11 @@ def classifier_callback(option, opt_str, value, parser):
 
 
 def main():
+    api_config.register_config()
+    cfg.CONF(args=[],
+             project='bagpipe-rest-attach',
+             default_config_files=['/etc/bagpipe-bgp/bgp.conf'])
+
     usage = "usage: %prog [--attach|--detach] --network-type (ipvpn|evpn) "\
         "--port (<port>|netns) --ip <ip>[/<mask>] [options] (see --help)"
     parser = optparse.OptionParser(usage)
@@ -255,18 +264,18 @@ def main():
                       default=None,
                       type="int",
                       help="BGP LOCAL PREF attribute (optional)")
-    (options, _) = parser.parse_args()
+    (options, _unused) = parser.parse_args()
 
-    if not(options.operation):
+    if not options.operation:
         parser.error("Need to specify --attach or --detach")
 
-    if not(options.port):
+    if not options.port:
         parser.error("Need to specify --port <localport>")
 
-    if not(options.network_type):
+    if not options.network_type:
         parser.error("Need to specify --network-type")
 
-    if not(options.ip):
+    if not options.ip:
         parser.error("Need to specify --ip")
 
     if (len(options.route_targets) == 0 and
@@ -288,7 +297,7 @@ def main():
     if not re.match('.*/[0-9]+$', options.ip):
         options.ip = options.ip + "/24"
 
-    if not(options.gw_ip):
+    if not options.gw_ip:
         net = netaddr.IPNetwork(options.ip)
         print("using %s as gateway address" % str(net[-2]))
         options.gw_ip = str(net[-2])
@@ -303,7 +312,7 @@ def main():
             options.netns = options.vpn_instance_id
 
         try:
-            (_, options.if2netns) = options.port.split(":")
+            (_unused, options.if2netns) = options.port.split(":")
         except Exception:
             options.if2netns = get_vpn2ns_if_name(options.netns)
 
@@ -377,14 +386,14 @@ def main():
                              "one static destination prefix option")
 
     data = {
-        "import_rt":  import_rts,
-        "export_rt":  export_rts,
-        "local_port":  local_port,
-        "vpn_instance_id":  options.vpn_instance_id,
-        "vpn_type":    options.network_type,
-        "gateway_ip":  options.gw_ip,
+        "import_rt": import_rts,
+        "export_rt": export_rts,
+        "local_port": local_port,
+        "vpn_instance_id": options.vpn_instance_id,
+        "vpn_type": options.network_type,
+        "gateway_ip": options.gw_ip,
         "mac_address": options.mac,
-        "ip_address":  options.ip,
+        "ip_address": options.ip,
         "advertise_subnet": options.advertise_subnet,
         "readvertise": readvertise,
         "attract_traffic": attract_traffic,
@@ -403,16 +412,17 @@ def main():
     print("request: %s" % json_data)
 
     os.environ['NO_PROXY'] = "127.0.0.1"
-    req = urllib2.Request("http://127.0.0.1:8082/%s_localport" %
-                          options.operation, json_data,
-                          {'Content-Type': 'application/json'})
+    req = urllib.request.Request(
+        "http://127.0.0.1:%d/%s_localport" %
+        (cfg.CONF.API.port, options.operation),
+        json_data, {'Content-Type': 'application/json'})
     try:
-        response = urllib2.urlopen(req)
+        response = urllib.request.urlopen(req)
         response_content = response.read()
         response.close()
 
         print("response: %d %s" % (response.getcode(), response_content))
-    except urllib2.HTTPError as e:
+    except urllib.error.HTTPError as e:
         error_content = e.read()
         print("   %s" % error_content)
         sys.exit("error %d, reason: %s" % (e.code, e.reason))
