@@ -1,6 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-# encoding: utf-8
-
 # Copyright 2018 Orange
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -24,10 +21,9 @@ from networking_bagpipe._i18n import _
 from networking_bagpipe.bagpipe_bgp.common import log_decorator
 
 from neutron.agent.common import ovs_lib
-from neutron.plugins.ml2.drivers.openvswitch.agent.common import \
-    constants as ovs_const
 
 from neutron_lib import exceptions
+from neutron_lib.plugins.ml2 import ovs_constants as ovs_const
 
 LOG = logging.getLogger(__name__)
 
@@ -134,7 +130,11 @@ def _build_group_expr_str(group_dict, cmd):
 
 class OVSExtendedBridge(ovs_lib.OVSBridge):
 
-    def add_flow_extended(self, flow_matches=[], actions=[]):
+    def add_flow_extended(self, flow_matches=None, actions=None):
+        if flow_matches is None:
+            flow_matches = []
+        if actions is None:
+            actions = []
         flow_args = {}
         for match in flow_matches:
             flow_args.update(match)
@@ -144,7 +144,9 @@ class OVSExtendedBridge(ovs_lib.OVSBridge):
 
         self.add_flow(**flow_args)
 
-    def delete_flows_extended(self, flow_matches=[]):
+    def delete_flows_extended(self, flow_matches=None):
+        if flow_matches is None:
+            flow_matches = []
         flow_args = {}
         for match in flow_matches:
             flow_args.update(match)
@@ -225,7 +227,7 @@ class ObjectLifecycleManager(object):
 
     @log_decorator.log_info
     def clear_objects(self, filter_method):
-        for object_key, users in self.object_used_for.items():
+        for object_key, users in list(self.object_used_for.items()):
             for user in users:
                 if filter_method(object_key, user):
                     self.delete_object(self.objects[object_key])
