@@ -1,6 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-# encoding: utf-8
-
 # Copyright 2014 Orange
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,10 +14,8 @@
 
 import abc
 import socket
-import traceback
 
 from oslo_log import log as logging
-import six
 
 from networking_bagpipe.bagpipe_bgp.common import log_decorator
 from networking_bagpipe.bagpipe_bgp.common import looking_glass as lg
@@ -97,10 +92,11 @@ def equivalent_route_in_routes(function, route, routes):
 
 def compare_ecmp(worker, route_a, route_b):
     # makes a comparison based on LOCAL_PREF
+    # if route has no LOCAL_PREF, use default value (100) as defined in exabgp
     l_a = route_a.attributes.get(exa.Attribute.CODE.LOCAL_PREF)
-    l_a = l_a.localpref if l_a else -1
+    l_a = l_a.localpref if l_a else 100
     l_b = route_b.attributes.get(exa.Attribute.CODE.LOCAL_PREF)
-    l_b = l_b.localpref if l_b else -1
+    l_b = l_b.localpref if l_b else 100
     return (l_a > l_b) - (l_b > l_a)
 
 
@@ -129,8 +125,8 @@ def compare_no_ecmp(worker, route_a, route_b):
     return (route_a > route_b) - (route_b > route_a)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class TrackerWorker(worker.Worker, lg.LookingGlassLocalLogger):
+class TrackerWorker(worker.Worker, lg.LookingGlassLocalLogger,
+                    metaclass=abc.ABCMeta):
 
     def __init__(self, bgp_manager, worker_name,
                  compare_routes=compare_no_ecmp):
@@ -206,7 +202,7 @@ class TrackerWorker(worker.Worker, lg.LookingGlassLocalLogger):
                     self.log.trace("All best routes have been replaced")
                     self._recompute_best_routes(all_routes, best_routes)
                     if best_routes:
-                        current_best = six.next(iter(best_routes))
+                        current_best = next(iter(best_routes))
                         self.log.trace("We'll need to call new_best_route for "
                                        "all our new best routes")
                         call_new_best_route_4_all = True
@@ -216,7 +212,7 @@ class TrackerWorker(worker.Worker, lg.LookingGlassLocalLogger):
                 else:
                     # (if there is more than one route in the best routes, we
                     # take the first one)
-                    current_best = six.next(iter(best_routes))
+                    current_best = next(iter(best_routes))
 
                     self.log.trace("Current best route: %s", current_best)
 
@@ -390,13 +386,15 @@ class TrackerWorker(worker.Worker, lg.LookingGlassLocalLogger):
             )
         )
 
-    def synthesize_withdraw_all(self):
+    def synthesize_withdraw_all(self, afi, safi):
         for tracked_entry, routes in list(self.tracked_entry_2_routes.items()):
-            self.log.trace("Synthethizing withdraws for all routes of %s",
-                           tracked_entry)
+            self.log.trace("Synthesizing withdraws for all routes of %s with "
+                           "AFI(%s)/SAFI(%s)", tracked_entry, afi, safi)
             for route in routes:
-                self._on_event(engine.RouteEvent(engine.RouteEvent.WITHDRAW,
-                                                 route))
+                if (route.nlri.afi, route.nlri.safi) == (afi, safi):
+                    self._on_event(
+                        engine.RouteEvent(engine.RouteEvent.WITHDRAW, route)
+                    )
 
     @log_decorator.log
     def _call_new_best_route_for_routes(self, entry, routes):
@@ -412,7 +410,7 @@ class TrackerWorker(worker.Worker, lg.LookingGlassLocalLogger):
         except Exception as e:
             self.log.error("Exception in <subclass>.new_best_route: %s", e)
             if self.log.isEnabledFor(logging.WARNING):
-                self.log.info("%s", traceback.format_exc())
+                self.log.exception("")
 
     @log_decorator.log
     def _call_best_route_removed(self, entry, old_route, last):
@@ -422,7 +420,7 @@ class TrackerWorker(worker.Worker, lg.LookingGlassLocalLogger):
             self.log.error("Exception in <subclass>.best_route_removed: %s",
                            e)
             if self.log.isEnabledFor(logging.WARNING):
-                self.log.info("%s", traceback.format_exc())
+                self.log.exception("")
 
     # Callbacks for subclasses ########################
 
@@ -464,7 +462,7 @@ class TrackerWorker(worker.Worker, lg.LookingGlassLocalLogger):
     def _dump_state(self):
         if self.log.isEnabledFor(logging.TRACE):
             self.log.trace("--- tracked_entry_2_routes ---")
-            for (entry, routes) in six.iteritems(self.tracked_entry_2_routes):
+            for (entry, routes) in self.tracked_entry_2_routes.items():
                 self.log.trace(
                     "  Entry: %s", TrackerWorker._display_entry(entry))
                 for route in routes:
@@ -472,7 +470,7 @@ class TrackerWorker(worker.Worker, lg.LookingGlassLocalLogger):
 
             self.log.trace("--- tracked_entry_2_best_routes ---")
             for (entry, best_routes) in \
-                    six.iteritems(self.tracked_entry_2_best_routes):
+                    self.tracked_entry_2_best_routes.items():
                 self.log.trace("  Entry: %s",
                                TrackerWorker._display_entry(entry))
                 for route in best_routes:
@@ -483,8 +481,7 @@ class TrackerWorker(worker.Worker, lg.LookingGlassLocalLogger):
     @staticmethod
     def _display_entry(entry):
         if (isinstance(entry, tuple) and len(entry) > 0 and
-                (isinstance(entry[0], type) or
-                 isinstance(entry[0], six.class_types))):
+                isinstance(entry[0], type)):
             return repr(tuple([entry[0].__name__] + list(entry[1:])))
         else:
             return repr(entry)
@@ -505,7 +502,7 @@ class TrackerWorker(worker.Worker, lg.LookingGlassLocalLogger):
 
     def _get_lg_routes(self, path_prefix, route_dict):
         routes = {}
-        for entry in six.iterkeys(route_dict):
+        for entry in route_dict.keys():
             entry_repr = self._display_entry(entry)
             routes[entry_repr] = [route.get_looking_glass_info(path_prefix)
                                   for route in route_dict[entry]]
