@@ -1,6 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-# encoding: utf-8
-
 # Copyright 2014 Orange
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,12 +13,10 @@
 # limitations under the License.
 
 import abc
-import traceback
 
-from distutils import version  # pylint: disable=no-name-in-module
 from oslo_config import cfg
 from oslo_log import log as logging
-import six
+from oslo_utils import versionutils
 import stevedore
 
 from networking_bagpipe.bagpipe_bgp.common import config
@@ -60,6 +55,7 @@ for vpn_type in constants.VPN_TYPES:
 def register_driver_opts(vpn_type, driver_opts):
     cfg.CONF.register_opts(driver_opts, constants.config_group(vpn_type))
 
+
 # prefix for setuptools entry points for dataplane drivers
 DATAPLANE_DRIVER_ENTRY_POINT_PFX = "bagpipe.dataplane"
 
@@ -87,17 +83,16 @@ def instantiate_dataplane_drivers():
             ).driver
 
             drivers[vpn_type] = driver_class()
-        except Exception as e:
-            LOG.error("Error while instantiating dataplane"
-                      " driver for %s with %s: %s", vpn_type, driver_name, e)
-            LOG.error(traceback.format_exc())
+        except Exception:
+            LOG.exception("Error while instantiating dataplane driver for "
+                          "%s with %s", vpn_type, driver_name)
             raise
 
     return drivers
 
 
-@six.add_metaclass(abc.ABCMeta)
-class DataplaneDriver(lg.LookingGlassLocalLogger):
+class DataplaneDriver(lg.LookingGlassLocalLogger, utils.ClassReprMixin,
+                      metaclass=abc.ABCMeta):
     '''Dataplane driver
 
     The initialisation workflow is the following:
@@ -151,8 +146,11 @@ class DataplaneDriver(lg.LookingGlassLocalLogger):
         o = self._run_command("uname -r")
         self.kernel_release = o[0][0].split("-")[0]
         if self.required_kernel:
-            if (version.StrictVersion(self.kernel_release) <
-                    version.StrictVersion(self.required_kernel)):
+            kernel_release_version = versionutils.convert_version_to_tuple(
+                self.kernel_release)
+            required_kernel_version = versionutils.convert_version_to_tuple(
+                self.required_kernel)
+            if kernel_release_version < required_kernel_version:
                 self.log.warning("%s requires at least Linux kernel %s"
                                  " (you are running %s)",
                                  self.__class__.__name__,
@@ -264,19 +262,18 @@ class DataplaneDriver(lg.LookingGlassLocalLogger):
         }
 
 
-@six.add_metaclass(abc.ABCMeta)
-class VPNInstanceDataplane(lg.LookingGlassLocalLogger):
+class VPNInstanceDataplane(lg.LookingGlassLocalLogger, metaclass=abc.ABCMeta):
 
     @log_decorator.log_info
     def __init__(self, dataplane_driver, instance_id, external_instance_id,
-                 gateway_ip, mask, instance_label=None, **kwargs):
+                 gateway_ip, network_plen, instance_label=None, **kwargs):
         lg.LookingGlassLocalLogger.__init__(self, repr(instance_id))
         self.driver = dataplane_driver
         self.config = dataplane_driver.config
         self.instance_id = instance_id
         self.external_instance_id = external_instance_id
         self.gateway_ip = gateway_ip
-        self.mask = mask
+        self.network_plen = network_plen
         self.instance_label = instance_label
 
     @abc.abstractmethod
@@ -295,8 +292,8 @@ class VPNInstanceDataplane(lg.LookingGlassLocalLogger):
 
     def update_fallback(self, fallback):
         if fallback is not None:
-            self.log.warning("fallback  specified (%s) but not supported by"
-                             " driver, ignoring", fallback)
+            self.log.warning("fallback specified (%s) but not supported by "
+                             "driver, ignoring", fallback)
 
     @abc.abstractmethod
     def setup_dataplane_for_remote_endpoint(self, prefix, remote_pe, label,
@@ -321,7 +318,11 @@ class VPNInstanceDataplane(lg.LookingGlassLocalLogger):
         return self.driver.needs_cleanup_assist()
 
     def _run_command(self, command, run_as_root=False, *args, **kwargs):
-        return self.driver._run_command(command, run_as_root, *args, **kwargs)
+        return run_command.run_command(self.log, command, run_as_root,
+                                       *args, **kwargs)
+
+    def __repr__(self):
+        return "%s<%d>" % (self.__class__.__name__, self.instance_id)
 
     # Looking glass info ####
 
