@@ -1,6 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-# encoding: utf-8
-
 # Copyright 2014 Orange
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,8 +14,6 @@
 
 import abc
 
-import six
-
 from networking_bagpipe.bagpipe_bgp.common import log_decorator
 from networking_bagpipe.bagpipe_bgp.common import looking_glass as lg
 from networking_bagpipe.bagpipe_bgp.common import utils
@@ -29,8 +24,8 @@ from networking_bagpipe.bagpipe_bgp.vpn import dataplane_drivers as dp_drivers
 from networking_bagpipe.bagpipe_bgp.vpn import vpn_instance
 
 
-@six.add_metaclass(abc.ABCMeta)
-class VPNInstanceDataplane(dp_drivers.VPNInstanceDataplane):
+class VPNInstanceDataplane(dp_drivers.VPNInstanceDataplane,
+                           metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def add_dataplane_for_bum_endpoint(self, remote_pe, dpid, nlri, encaps):
@@ -110,8 +105,8 @@ class EVI(vpn_instance.VPNInstance, lg.LookingGlassMixin):
         self.gw_port = None
 
         encaps = self.dp_driver.supported_encaps()
-        if (exa.Encapsulation(exa.Encapsulation.Type.VXLAN) in encaps
-                and any([
+        if (exa.Encapsulation(exa.Encapsulation.Type.VXLAN) in encaps and
+                any([
                 exa.Encapsulation(exa.Encapsulation.Type.MPLS) in encaps,
                 exa.Encapsulation(exa.Encapsulation.Type.GRE) in encaps,
                 exa.Encapsulation(exa.Encapsulation.Type.MPLS_UDP) in encaps
@@ -150,7 +145,8 @@ class EVI(vpn_instance.VPNInstance, lg.LookingGlassMixin):
     def generate_vif_bgp_route(self, mac_address, ip_prefix, plen, label, rd):
         # Generate BGP route and advertise it...
 
-        assert(plen == 32)
+        if ip_prefix:
+            assert plen == 32
 
         if self._vxlan_dp_driver():
             mpls_label_field = exa.Labels([], raw_labels=[self.instance_label])
@@ -159,9 +155,9 @@ class EVI(vpn_instance.VPNInstance, lg.LookingGlassMixin):
 
         # label parameter ignored, we need to use instance label
         nlri = exa.EVPNMAC(
-            rd, exa.ESI(), exa.EthernetTag(), exa.MAC(mac_address), 6*8,
+            rd, exa.ESI(), exa.EthernetTag(), exa.MAC(mac_address), 6 * 8,
             mpls_label_field,
-            exa.IP.create(ip_prefix), None,
+            exa.IP.create(ip_prefix) if ip_prefix else None, None,
             exa.IP.create(self.dp_driver.get_local_address()))
 
         return engine.RouteEntry(nlri)
