@@ -1,6 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-# encoding: utf-8
-
 # Copyright 2014 Orange
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -25,7 +22,6 @@ import threading
 
 import netaddr
 from oslo_log import log as logging
-import six
 
 from networking_bagpipe.bagpipe_bgp.common import exceptions as exc
 from networking_bagpipe.bagpipe_bgp.common import log_decorator
@@ -107,9 +103,9 @@ class TrafficClassifier(object):
             elif op == '<=':
                 port_range = (0, int(port))
             elif op == '>':
-                port_range = (int(port)+1, 65535)
+                port_range = (int(port) + 1, 65535)
             elif op == '<':
-                port_range = (0, int(port)-1)
+                port_range = (0, int(port) - 1)
             else:
                 port_range = int(port)
         else:
@@ -120,9 +116,9 @@ class TrafficClassifier(object):
                 if op == '>=' or op == '<=':
                     port_range += int(port)
                 elif op == '>':
-                    port_range += int(port)+1
+                    port_range += int(port) + 1
                 elif op == '<':
-                    port_range += int(port)-1
+                    port_range += int(port) - 1
 
         return port_range
 
@@ -133,7 +129,7 @@ class TrafficClassifier(object):
         port_rules = list()
 
         # Check if port range or port number
-        if type(port_range) == tuple:
+        if isinstance(port_range, tuple):
             port_min, port_max = port_range
 
             if int(port_min) == 0:
@@ -233,14 +229,14 @@ class TrafficClassifier(object):
                       5: self._parse_destination_port,  # FLowDestinationPort
                       6: self._parse_source_port}  # FlowSourcePort
 
-        for ID, rule in six.iteritems(rules):
+        for ID, rule in rules.items():
             components[ID](rule)
 
 
-@six.add_metaclass(abc.ABCMeta)
 class VPNInstance(tracker_worker.TrackerWorker,
                   threading.Thread,
-                  lg.LookingGlassLocalLogger):
+                  lg.LookingGlassLocalLogger,
+                  metaclass=abc.ABCMeta):
 
     type = None  # set by subclasses: 'ipvpn', 'evpn', etc.
     afi = None
@@ -249,8 +245,8 @@ class VPNInstance(tracker_worker.TrackerWorker,
     @log_decorator.log
     def __init__(self, vpn_manager, dataplane_driver,
                  external_instance_id, instance_id, import_rts, export_rts,
-                 gateway_ip, mask, readvertise, attract_traffic, fallback=None,
-                 **kwargs):
+                 gateway_ip, ip_address_plen, readvertise, attract_traffic,
+                 fallback=None, **kwargs):
 
         self.manager = vpn_manager
 
@@ -269,20 +265,17 @@ class VPNInstance(tracker_worker.TrackerWorker,
 
         tracker_worker.TrackerWorker.__init__(
             self,
-            self.manager.bgp_manager, "%s-%d" % (self.instance_type,
-                                                 self.instance_id),
+            self.manager.bgp_manager, repr(self),
             compare_routes)
 
-        lg.LookingGlassLocalLogger.__init__(self,
-                                            "%s-%d" % (self.instance_type,
-                                                       self.instance_id))
+        lg.LookingGlassLocalLogger.__init__(self, repr(self))
         self.lock = threading.RLock()
 
         self.import_rts = import_rts
         self.export_rts = export_rts
         self.external_instance_id = external_instance_id
         self.gateway_ip = gateway_ip
-        self.mask = mask
+        self.network_plen = ip_address_plen
 
         self.fallback = None
 
@@ -296,13 +289,11 @@ class VPNInstance(tracker_worker.TrackerWorker,
             self.forced_vni = True
         else:
             self.instance_label = self.manager.label_allocator.get_new_label(
-                "Incoming traffic for %s %d" % (self.instance_type,
-                                                self.instance_id))
+                "Incoming traffic for %s" % self)
             self.forced_vni = False
 
         self.instance_rd = self.manager.rd_allocator.get_new_rd(
-            "Default route distinguisher for %s %d" % (self.instance_type,
-                                                       self.instance_id))
+            "Default route distinguisher for %s" % self)
 
         self.localport_data = dict()
 
@@ -314,7 +305,7 @@ class VPNInstance(tracker_worker.TrackerWorker,
         self.endpoint_2_route = dict()
         # One MAC address -> One local port
         self.mac_2_localport_data = dict()
-        # One IP address ->  Multiple MAC address
+        # One IP address prefix ->  Multiple MAC address
         self.ip_address_2_mac = collections.defaultdict(set)
 
         # One endpoint (MAC and IP addresses tuple) -> BGP local_pref
@@ -326,13 +317,16 @@ class VPNInstance(tracker_worker.TrackerWorker,
         # endpoint (MAC and IP addresses tuple) -> direction
         self.endpoint_2_direction = dict()
 
+        # MAC -> set of IP addresses
+        self.mac_2_ips = collections.defaultdict(set)
+
         # Redirected instances list from which traffic is attracted (based on
         # FlowSpec 5-tuple classification)
         self.redirected_instances = set()
 
         self.dataplane = self.dp_driver.initialize_dataplane_instance(
             self.instance_id, self.external_instance_id,
-            self.gateway_ip, self.mask, self.instance_label, **kwargs)
+            self.gateway_ip, self.network_plen, self.instance_label, **kwargs)
 
         for rt in self.import_rts:
             self._subscribe(self.afi, self.safi, rt)
@@ -406,14 +400,26 @@ class VPNInstance(tracker_worker.TrackerWorker,
 
         self.dataplane.update_fallback(fallback)
 
+    def needs_cleanup_assist(self, afi, safi):
+        return self.dataplane.needs_cleanup_assist()
+
+    def __repr__(self):
+        return "%s-%s" % (self.instance_type, self.instance_id)
+
     @utils.synchronized
     @log_decorator.log
     def stop(self):
+        if self._was_stopped:
+            LOG.debug("was already stopped, nothing to do to stop")
+            return
         self.stop_event_loop()
 
-        if self.dataplane.needs_cleanup_assist():
-            self.log.debug("Dataplane driver needs cleanup assistance")
-            self.synthesize_withdraw_all()
+        for afi in (self.afi,):
+            for safi in (self.safi, exa.SAFI.flow_vpn):
+                if self.needs_cleanup_assist(afi, safi):
+                    self.log.debug("Dataplane driver needs cleanup assistance "
+                                   "for AFI(%s)/SAFI(%s)", afi, safi)
+                    self.synthesize_withdraw_all(afi, safi)
 
         self.dataplane.cleanup()
         if not self.forced_vni:
@@ -441,7 +447,7 @@ class VPNInstance(tracker_worker.TrackerWorker,
 
     def has_only_one_endpoint(self):
         return (len(self.localport_2_endpoints) == 1 and
-                len(six.next(six.itervalues(self.localport_2_endpoints))) == 1)
+                len(next(iter(self.localport_2_endpoints.values()))) == 1)
 
     def all_endpoints(self):
         return itertools.chain(*self.localport_2_endpoints.values())
@@ -451,10 +457,8 @@ class VPNInstance(tracker_worker.TrackerWorker,
         added_import_rt = set(new_import_rts) - set(self.import_rts)
         removed_import_rt = set(self.import_rts) - set(new_import_rts)
 
-        self.log.debug("%s %d - Added Import RTs: %s",
-                       self.instance_type, self.instance_id, added_import_rt)
-        self.log.debug("%s %d - Removed Import RTs: %s",
-                       self.instance_type, self.instance_id, removed_import_rt)
+        self.log.debug("Added Import RTs: %s", added_import_rt)
+        self.log.debug("Removed Import RTs: %s", removed_import_rt)
 
         # Unregister from BGP with these route targets
         for rt in removed_import_rt:
@@ -503,15 +507,10 @@ class VPNInstance(tracker_worker.TrackerWorker,
             self.dataplane.update_fallback(fallback)
 
     def _parse_ipaddress_prefix(self, ip_address_prefix):
-        ip_address = ""
-        mask = 0
-        try:
-            net = netaddr.IPNetwork(ip_address_prefix)
-            (ip_address, mask) = (str(net.ip), net.prefixlen)
-        except netaddr.core.AddrFormatError:
-            raise exc.APIException("Bogus IP prefix: %s" % ip_address_prefix)
-
-        return (ip_address, mask)
+        if ip_address_prefix is None:
+            return (None, 0)
+        net = netaddr.IPNetwork(ip_address_prefix)
+        return (str(net.ip), net.prefixlen)
 
     def _gen_encap_extended_communities(self):
         ecommunities = exa.extcoms.ExtendedCommunities()
@@ -549,8 +548,8 @@ class VPNInstance(tracker_worker.TrackerWorker,
             exa.ConsistentHashSortOrder(lb_consistent_hash_order))
         route_entry.attributes.add(ecommunities)
 
-        route_entry.attributes.add(exa.LocalPreference(local_pref
-                                                       or DEFAULT_LOCAL_PREF))
+        route_entry.attributes.add(exa.LocalPreference(local_pref or
+                                                       DEFAULT_LOCAL_PREF))
 
         self.log.debug("Synthesized Vif route entry: %s", route_entry)
         return route_entry
@@ -584,15 +583,14 @@ class VPNInstance(tracker_worker.TrackerWorker,
     @classmethod
     def validate_convert_params(cls, params, also_mandatory=()):
 
-        for param in ('vpn_instance_id', 'mac_address', 'ip_address',
+        for param in ('vpn_instance_id', 'mac_address',
                       'local_port') + also_mandatory:
             if param not in params:
                 raise exc.APIMissingParameterException(param)
 
         # if local_port is not a dict, then assume it designates a linux
         # interface
-        if (isinstance(params['local_port'], six.string_types) or
-                isinstance(params['local_port'], six.text_type)):
+        if isinstance(params['local_port'], str):
             params['local_port'] = {'linuxif': params['local_port']}
 
         for param in ('import_rt', 'export_rt'):
@@ -600,8 +598,7 @@ class VPNInstance(tracker_worker.TrackerWorker,
                 continue
 
             # if import_rt or export_rt are strings, convert them into lists
-            if (isinstance(params[param], six.string_types) or
-                    isinstance(params[param], six.text_type)):
+            if isinstance(params[param], str):
                 try:
                     params[param] = re.split(',+ *', params[param])
                 except Exception:
@@ -615,17 +612,30 @@ class VPNInstance(tracker_worker.TrackerWorker,
             raise exc.APIException("Mandatory key is missing in local_port "
                                    "parameter (linuxif, or evpn)")
 
-        # Verify and format IP address with prefix if necessary
-        if re.match(r'([12]?\d?\d\.){3}[12]?\d?\d\/[123]?\d',
-                    params['ip_address']):
-            params['ip_address_prefix'] = params['ip_address']
-        elif re.match(r'([12]?\d?\d\.){3}[12]?\d?\d', params['ip_address']):
-            params['ip_address_prefix'] = params['ip_address'] + "/32"
-        else:
-            raise exc.MalformedIPAddress(params['ip_address'])
+        # validate mac_address
+        try:
+            netaddr.EUI(params['mac_address'])
+        except netaddr.core.AddrFormatError:
+            raise exc.MalformedMACAddress(params['mac_address'])
 
-        # NOTE(tmorin): improve the IP check above and validate MAC address
-        # format
+        # validate ip_address if present
+        if 'ip_address' in params:
+            try:
+                net = netaddr.IPNetwork(params['ip_address'])
+                params['ip_address_prefix'] = params['ip_address']
+                params['ip_address_plen'] = net.prefixlen
+            except netaddr.core.AddrFormatError:
+                # Try again assuming ip_address is an IP without a prefix
+                # (implicitly using /32)
+                try:
+                    netaddr.IPAddress(params['ip_address'])
+                    params['ip_address_prefix'] = params['ip_address'] + "/32"
+                    params['ip_address_plen'] = 32
+                except netaddr.core.AddrFormatError:
+                    raise exc.MalformedIPAddress(params['ip_address'])
+        else:
+            params['ip_address_prefix'] = None
+            params['ip_address_plen'] = None
 
         if not isinstance(params.get('advertise_subnet', False), bool):
             raise exc.APIException("'advertise_subnet' must be a boolean")
@@ -650,7 +660,7 @@ class VPNInstance(tracker_worker.TrackerWorker,
         cls.validate_convert_params(
             params,
             also_mandatory=('import_rt', 'export_rt')
-            )
+        )
 
         params['advertise_subnet'] = params.get('advertise_subnet', False)
         params['lb_consistent_hash_order'] = params.get(
@@ -677,6 +687,39 @@ class VPNInstance(tracker_worker.TrackerWorker,
             self.endpoint_2_rd[endpoint] = rd
         return rd
 
+    def _raise_if_mac2ip_inconsistency(self, mac_address, ip_address_prefix):
+        if not ip_address_prefix:
+            return
+
+        if mac_address not in self.ip_address_2_mac[ip_address_prefix]:
+            raise exc.APIException(
+                "Inconsistent endpoint info: IP %s already bound to a MAC "
+                "address different from %s (%s)" %
+                (ip_address_prefix, mac_address,
+                 self.ip_address_2_mac[ip_address_prefix]))
+
+    def _check_ip_mac(self, mac_address, ip_address_prefix, advertise_subnet):
+        (ip_prefix, plen) = self._parse_ipaddress_prefix(ip_address_prefix)
+
+        # Special case where no IP was provided
+        if ip_prefix is None:
+            refresh_only = mac_address in self.mac_2_localport_data
+            return (None, None, refresh_only)
+
+        if not advertise_subnet and plen != 32:
+            self.log.debug("Using /32 instead of /%s", plen)
+            plen = 32
+
+        # - Verify (MAC address, IP address) tuple consistency
+        refresh_only = False
+        if ip_address_prefix in self.ip_address_2_mac and plen == 32:
+            self._raise_if_mac2ip_inconsistency(mac_address, ip_address_prefix)
+
+            LOG.debug("IP/MAC already plugged, only updating route")
+            refresh_only = True
+
+        return ip_prefix, plen, refresh_only
+
     @utils.synchronized
     @log_decorator.log_info
     def vif_plugged(self, mac_address, ip_address_prefix, localport,
@@ -702,35 +745,16 @@ class VPNInstance(tracker_worker.TrackerWorker,
         try:
             self.endpoint_2_desc[endpoint] = kwargs.get('description')
 
-            # Parse address/mask
-            (ip_prefix, plen) = self._parse_ipaddress_prefix(ip_address_prefix)
-
-            if not advertise_subnet and plen != 32:
-                self.log.debug("Using /32 instead of /%d", plen)
-                plen = 32
-
-            # - Verify (MAC address, IP address) tuple consistency
-            refresh_only = False
-            if ip_address_prefix in self.ip_address_2_mac and plen == 32:
-                if mac_address not in self.ip_address_2_mac[ip_address_prefix]:
-                    raise exc.APIException("Inconsistent endpoint info: %s "
-                                           "already bound to a MAC address "
-                                           "different from %s" %
-                                           (ip_address_prefix, mac_address))
-                else:
-                    LOG.debug("IP/MAC already plugged, only updating route")
-                    refresh_only = True
-                    return
+            (ip_prefix, plen, refresh_only) = self._check_ip_mac(
+                mac_address, ip_address_prefix, advertise_subnet)
 
             self.log.debug("Plugging port (%s)", ip_prefix)
 
             pdata = self.mac_2_localport_data.get(mac_address, dict())
             if not pdata:
                 pdata['label'] = self.manager.label_allocator.get_new_label(
-                    "Incoming traffic for %s %d, interface %s"
-                    ", endpoint %s/%s" %
-                    (self.instance_type, self.instance_id,
-                     linuxif, mac_address, ip_address_prefix)
+                    "Incoming traffic for %s, interface %s, endpoint %s" %
+                    (self, linuxif, endpoint)
                 )
                 pdata["port_info"] = localport
                 pdata["lb_consistent_hash_order"] = lb_consistent_hash_order
@@ -747,17 +771,14 @@ class VPNInstance(tracker_worker.TrackerWorker,
             if forward_to_port(direction):
                 endpoint_rd = self._rd_for_endpoint(
                     endpoint,
-                    "Route distinguisher for %s %d, interface %s, "
-                    "endpoint %s/%s" % (self.instance_type, self.instance_id,
-                                        linuxif, mac_address,
-                                        ip_address_prefix)
+                    "Route distinguisher for %s, interface %s, endpoint %s" %
+                    (self, linuxif, endpoint)
                 )
 
                 rd = self.instance_rd if plen == 32 else endpoint_rd
 
                 self.log.info("Synthesizing and advertising BGP route for VIF "
-                              "%s endpoint (%s, %s/%d)", linuxif,
-                              mac_address, ip_prefix, plen)
+                              "%s endpoint %s", linuxif, endpoint)
                 route_entry = self.synthesize_vif_bgp_route(
                     mac_address, ip_prefix, plen,
                     pdata['label'], lb_consistent_hash_order, rd, local_pref)
@@ -771,7 +792,21 @@ class VPNInstance(tracker_worker.TrackerWorker,
             self.endpoint_2_direction[endpoint] = direction
             self.mac_2_localport_data[mac_address] = pdata
 
-            self.ip_address_2_mac[ip_address_prefix].add(mac_address)
+            if ip_address_prefix:
+                self.ip_address_2_mac[ip_address_prefix].add(mac_address)
+                self.mac_2_ips[mac_address].add(ip_address_prefix)
+
+                # we've a non-wildcard plug, so if a wildcard IP was plugged
+                # for this MAC, consider it replaced:
+                # - withdraw the corresponding route
+                # - cleanup the wildcard endpoint records
+                wildcard_endpoint = (mac_address, None)
+                if wildcard_endpoint in self.localport_2_endpoints[linuxif]:
+                    self._withdraw_route(self.endpoint_2_route.pop(
+                        wildcard_endpoint))
+                    self.localport_2_endpoints[linuxif].remove(
+                        wildcard_endpoint)
+                    del self.endpoint_2_desc[wildcard_endpoint]
 
         except Exception as e:
             self.log.error("Error in vif_plugged: %s", e)
@@ -786,29 +821,52 @@ class VPNInstance(tracker_worker.TrackerWorker,
 
             raise
 
-        self.log.info("localport_2_endpoints: %s", self.localport_2_endpoints)
-        self.log.info("endpoint_2_rd: %s", self.endpoint_2_rd)
-        self.log.info("mac_2_localport_data: %s", self.mac_2_localport_data)
-        self.log.info("ip_address_2_mac: %s", self.ip_address_2_mac)
+        self.log.debug("localport_2_endpoints: %s", self.localport_2_endpoints)
+        self.log.debug("endpoint_2_rd: %s", self.endpoint_2_rd)
+        self.log.debug("mac_2_localport_data: %s", self.mac_2_localport_data)
+        self.log.debug("ip_address_2_mac: %s", self.ip_address_2_mac)
 
     @utils.synchronized
     @log_decorator.log_info
     def vif_unplugged(self, mac_address, ip_address_prefix):
+        if ip_address_prefix is None:
+            # wildcard IP address case...
+            errors = False
+            for ip_addr_pfx in list(self.mac_2_ips[mac_address]):
+                try:
+                    self.vif_unplugged_real(mac_address, ip_addr_pfx)
+                except Exception:
+                    LOG.exception("Exception while unplugging (%s, %s)",
+                                  mac_address, ip_addr_pfx)
+                    errors = True
+            if errors:
+                raise Exception("There were errors on at least one of the "
+                                "unplug resulting from the wildcard unplug")
+            # remove the wildcard endpoint itself, if any:
+            if (mac_address, None) in self.endpoint_2_rd:
+                try:
+                    self.vif_unplugged_real(mac_address, None)
+                except exc.APINotPluggedYet:
+                    LOG.debug("Wildcard unplug failed because not-plugged-yet,"
+                              " ignoring.")
+        else:
+            self.vif_unplugged_real(mac_address, ip_address_prefix)
+
+    @log_decorator.log_info
+    def vif_unplugged_real(self, mac_address, ip_address_prefix):
         # NOTE(tmorin): move this as a vif_unplugged_precheck, so that
         # in ipvpn.VRF this is done before readvertised route withdrawal
         endpoint = (mac_address, ip_address_prefix)
         # Verify port and endpoint (MAC address, IP address) tuple consistency
         pdata = self.mac_2_localport_data.get(mac_address)
-        if (not pdata or
-                (ip_address_prefix in self.ip_address_2_mac and
-                 mac_address not in self.ip_address_2_mac[ip_address_prefix])
-                or endpoint not in self.endpoint_2_rd):
-            self.log.error("vif_unplugged called for endpoint (%s, %s), but "
-                           "no consistent informations or was not plugged yet",
-                           *endpoint)
-            raise exc.APIException("Inconsistent endpoint (%s, %s) info "
-                                   "or endpoint wasn't plugged yet, "
-                                   "cannot unplug" % endpoint)
+
+        if not pdata:
+            raise exc.APINotPluggedYet(endpoint)
+
+        self._raise_if_mac2ip_inconsistency(mac_address, ip_address_prefix)
+
+        if endpoint not in self.endpoint_2_rd:
+            raise Exception("Endpoint record missing: %s" % (endpoint,))
 
         # Finding label and local port informations
         label = pdata.get('label')
@@ -823,10 +881,11 @@ class VPNInstance(tracker_worker.TrackerWorker,
 
         if linuxif in self.localport_2_endpoints:
             # Parse address/mask
-            (ip_prefix, _) = self._parse_ipaddress_prefix(ip_address_prefix)
+            (ip_prefix, _unused) = self._parse_ipaddress_prefix(
+                ip_address_prefix)
 
-            self.log.info("Withdrawing BGP route for VIF %s endpoint "
-                          "(%s, %s)", linuxif, mac_address, ip_address_prefix)
+            self.log.info("Withdrawing BGP route for VIF %s endpoint %s",
+                          linuxif, endpoint)
 
             self._withdraw_route(self.endpoint_2_route.pop(endpoint))
 
@@ -862,21 +921,25 @@ class VPNInstance(tracker_worker.TrackerWorker,
             else:
                 del self.mac_2_localport_data[mac_address]
 
-            self.ip_address_2_mac[ip_address_prefix].discard(mac_address)
+            if ip_address_prefix:
+                self.ip_address_2_mac[ip_address_prefix].discard(mac_address)
+                if not self.ip_address_2_mac[ip_address_prefix]:
+                    del self.ip_address_2_mac[ip_address_prefix]
 
-            if not self.ip_address_2_mac[ip_address_prefix]:
-                del self.ip_address_2_mac[ip_address_prefix]
+                self.mac_2_ips[mac_address].discard(ip_address_prefix)
+                if not self.mac_2_ips[mac_address]:
+                    del self.mac_2_ips[mac_address]
 
             del self.endpoint_2_desc[endpoint]
         else:
-            self.log.error("vif_unplugged called for endpoint {%s, %s}, but"
-                           " port data is incomplete", mac_address,
-                           ip_address_prefix)
+            self.log.error("vif_unplugged called for endpoint %s, but"
+                           " port data is incomplete", endpoint)
             raise Exception("bagpipe-bgp bug, check its logs")
 
-        self.log.info("localport_2_endpoints: %s", self.localport_2_endpoints)
-        self.log.info("mac_2_localport_data: %s", self.mac_2_localport_data)
-        self.log.info("ip_address_2_mac: %s", self.ip_address_2_mac)
+        self.log.debug("localport_2_endpoints: %s", self.localport_2_endpoints)
+        self.log.debug("endpoint_2_rd: %s", self.endpoint_2_rd)
+        self.log.debug("mac_2_localport_data: %s", self.mac_2_localport_data)
+        self.log.debug("ip_address_2_mac: %s", self.ip_address_2_mac)
 
     @utils.synchronized
     def register_redirected_instance(self, instance_id):
@@ -1013,18 +1076,18 @@ class VPNInstance(tracker_worker.TrackerWorker,
 
     def get_lg_map(self):
         return {
-            "instance_type":         (lg.VALUE, self.instance_type),
-            "external_instance_id":  (lg.VALUE, self.external_instance_id),
-            "description":           (lg.VALUE, self.description),
-            "dataplane":             (lg.DELEGATE, self.dataplane),
-            "route_targets":         (lg.SUBITEM, self.get_rts),
-            "gateway_ip":            (lg.VALUE, self.gateway_ip),
-            "subnet_mask":           (lg.VALUE, self.mask),
+            "instance_type": (lg.VALUE, self.instance_type),
+            "external_instance_id": (lg.VALUE, self.external_instance_id),
+            "description": (lg.VALUE, self.description),
+            "dataplane": (lg.DELEGATE, self.dataplane),
+            "route_targets": (lg.SUBITEM, self.get_rts),
+            "gateway_ip": (lg.VALUE, self.gateway_ip),
+            "subnet_prefix_length": (lg.VALUE, self.network_plen),
             "instance_dataplane_id": (lg.VALUE, self.instance_label),
-            "ports":                 (lg.SUBTREE, self.get_lg_local_port_data),
-            "readvertise":           (lg.SUBITEM, self.get_lg_readvertise),
-            "attract_traffic":       (lg.SUBITEM, self.get_lg_attract_traffic),
-            "fallback":              (lg.VALUE, self.fallback)
+            "ports": (lg.SUBTREE, self.get_lg_local_port_data),
+            "readvertise": (lg.SUBITEM, self.get_lg_readvertise),
+            "attract_traffic": (lg.SUBITEM, self.get_lg_attract_traffic),
+            "fallback": (lg.VALUE, self.fallback)
         }
 
     def get_lg_local_port_data(self, path_prefix):
