@@ -1,6 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-# encoding: utf-8
-
 # Copyright 2014 Orange
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,9 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import six
-import testtools
-
 from oslo_config import fixture as config_fixture
 
 from networking_bagpipe.bagpipe_bgp.api import api
@@ -32,7 +26,6 @@ class TestAPI(base.TestCase):
         cfg_fixture = self.useFixture(config_fixture.Config())
         cfg_fixture.register_opts(api_config.common_opts, "API")
 
-    @testtools.skipIf(six.PY3, 'dataplane driver init fails under py3')
     def test_api_init(self):
         # instantiate the API, will fail if an exception is raised
         api.PecanAPI()
