@@ -1,6 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-# encoding: utf-8
-
 # Copyright 2014 Orange
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -38,8 +35,8 @@
 
 """
 
-import mock
 import testtools
+from unittest import mock
 
 from networking_bagpipe.bagpipe_bgp.common import exceptions as exc
 from networking_bagpipe.bagpipe_bgp import engine
@@ -48,9 +45,8 @@ from networking_bagpipe.bagpipe_bgp.engine import flowspec
 from networking_bagpipe.bagpipe_bgp.engine import ipvpn as ipvpn_routes
 from networking_bagpipe.bagpipe_bgp.engine import worker
 from networking_bagpipe.bagpipe_bgp.vpn import dataplane_drivers
+from networking_bagpipe.bagpipe_bgp.vpn import identifier_allocators
 from networking_bagpipe.bagpipe_bgp.vpn import ipvpn
-from networking_bagpipe.bagpipe_bgp.vpn import label_allocator
-from networking_bagpipe.bagpipe_bgp.vpn import rd_allocator
 from networking_bagpipe.bagpipe_bgp.vpn import vpn_instance
 from networking_bagpipe.tests.unit.bagpipe_bgp import base as t
 
@@ -163,7 +159,6 @@ class TestVPNInstanceAPIChecks(testtools.TestCase):
         method = vpn_instance.VPNInstance.validate_convert_attach_params
         self._test_validate_convert_missing(method, 'vpn_instance_id')
         self._test_validate_convert_missing(method, 'mac_address')
-        self._test_validate_convert_missing(method, 'ip_address')
         self._test_validate_convert_missing(method, 'local_port')
         self._test_validate_convert_missing(method, 'import_rt')
         self._test_validate_convert_missing(method, 'export_rt')
@@ -172,7 +167,6 @@ class TestVPNInstanceAPIChecks(testtools.TestCase):
         method = vpn_instance.VPNInstance.validate_convert_detach_params
         self._test_validate_convert_missing(method, 'vpn_instance_id')
         self._test_validate_convert_missing(method, 'mac_address')
-        self._test_validate_convert_missing(method, 'ip_address')
         self._test_validate_convert_missing(method, 'local_port')
 
     def test_api_internal_translation(self):
@@ -219,15 +213,31 @@ class TestVPNInstanceAPIChecks(testtools.TestCase):
             vpn_instance.VPNInstance.validate_convert_attach_params,
             params)
 
+    def test_mac_address_bogus(self):
+        params = api_params()
+        params['mac_address'] = 'gg:gg:gg:gg:gg:gg'
+        self.assertRaises(
+            exc.MalformedMACAddress,
+            vpn_instance.VPNInstance.validate_convert_attach_params,
+            params)
+
+    def test_ip_address_bogus(self):
+        params = api_params()
+        params['ip_address'] = '257.303.1.'
+        self.assertRaises(
+            exc.MalformedIPAddress,
+            vpn_instance.VPNInstance.validate_convert_attach_params,
+            params)
+
 
 class TestInitVPNInstance(testtools.TestCase):
 
     def setUp(self):
         super(TestInitVPNInstance, self).setUp()
-        self.mock_manager = mock.mock.Mock()
-        self.mock_manager.label_allocator.release = mock.mock.Mock()
-        self.mock_dp_driver = mock.mock.Mock()
-        self.mock_dp_driver.initialize_dataplane_instance = mock.mock.Mock()
+        self.mock_manager = mock.Mock()
+        self.mock_manager.label_allocator.release = mock.Mock()
+        self.mock_dp_driver = mock.Mock()
+        self.mock_dp_driver.initialize_dataplane_instance = mock.Mock()
 
     def test_init_stop_VPNInstance_with_forced_vni(self):
         # Initialize a VPNInstance with a forced VNID > 0
@@ -271,7 +281,7 @@ class TestVPNInstance(t.BaseTestBagPipeBGP, testtools.TestCase):
     def setUp(self):
         super(TestVPNInstance, self).setUp()
 
-        self.mock_dataplane = mock.mock.Mock(
+        self.mock_dataplane = mock.Mock(
             spec=dataplane_drivers.VPNInstanceDataplane)
 
         mock_dp_driver = mock.Mock(
@@ -336,9 +346,9 @@ class TestVPNInstance(t.BaseTestBagPipeBGP, testtools.TestCase):
                 endpoint,
                 self.vpn.localport_2_endpoints[localport['linuxif']])
 
-    def validate_convert_params_duplicate_rts(self):
+    def test_validate_convert_params_duplicate_rts(self):
         test_params = {'vpn_instance_id': 'foo',
-                       'mac_address': 'foo',
+                       'mac_address': 'aa:bb:cc:dd:ee:ff',
                        'ip_address': '1.2.3.4',
                        'local_port': 'foo',
                        'import_rt': ['64512:1', '64512:1'],
@@ -355,7 +365,6 @@ class TestVPNInstance(t.BaseTestBagPipeBGP, testtools.TestCase):
         self.vpn.vif_plugged(MAC1, IP1, LOCAL_PORT1)
 
         self.vpn.dataplane.vif_plugged.assert_called_once()
-        self.vpn._advertise_route.assert_called_once()
 
         self._validate_ip_address_2_mac_address_consistency(MAC1, IP1)
         self._chk_mac_2_localport_data_consistency(MAC1, LOCAL_PORT1)
@@ -716,6 +725,26 @@ class TestVPNInstance(t.BaseTestBagPipeBGP, testtools.TestCase):
         self.assertEqual({}, self.vpn.ip_address_2_mac)
         self.assertEqual({}, self.vpn.localport_2_endpoints)
 
+    def test_plug_unplug_wildcard_ip(self):
+        self.vpn.vif_plugged(MAC1, None, LOCAL_PORT1)
+        self.vpn.vif_plugged(MAC1, IP1, LOCAL_PORT1)
+        self.vpn.vif_plugged(MAC1, IP2, LOCAL_PORT1)
+
+        # 3 advertisements should be seen: one without IP, then one for each IP
+        self.assertEqual(3, self.vpn._advertise_route.call_count)
+        # the wildcard should be removed
+        self.assertEqual(1, self.vpn._withdraw_route.call_count)
+
+        self.vpn.vif_unplugged(MAC1, None)
+        # 3 withdraw should be seen, one for each IP, one without IP
+        self.assertEqual(3, self.vpn._withdraw_route.call_count)
+
+    def test_plug_unplug_wildcard_ip_no_ip(self):
+        self.vpn.vif_plugged(MAC1, None, LOCAL_PORT1)
+        self.assertEqual(1, self.vpn._advertise_route.call_count)
+        self.vpn.vif_unplugged(MAC1, None)
+        self.assertEqual(1, self.vpn._withdraw_route.call_count)
+
     def test_get_lg_localport_data(self):
         self.vpn.vif_plugged(MAC1, IP1, LOCAL_PORT1)
         self.vpn.vif_plugged(MAC2, IP2, LOCAL_PORT1)
@@ -813,8 +842,6 @@ IP_ADDR_PREFIX3 = '3.3.3.3/32'
 STATIC_ADDR_PREFIX1 = '10.10.10.10/32'
 STATIC_ADDR_PREFIX2 = '20.20.20.20/32'
 
-DEFAULT_ADDR_PREFIX = '0.0.0.0/0'
-
 ATTRACT_TRAFFIC_1 = {'redirect_rts': [t.RT5],
                      'classifier': {'destinationPort': '80',
                                     'protocol': 'tcp'
@@ -864,7 +891,7 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
     def setUp(self):
         super(TestVRF, self).setUp()
 
-        self.mock_dp = mock.mock.Mock(
+        self.mock_dp = mock.Mock(
             spec=ipvpn.DummyVPNInstanceDataplane)
 
         mock_dp_driver = mock.Mock(
@@ -876,10 +903,12 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
         mock_dp_driver.supported_encaps.return_value = \
             [exa.Encapsulation(exa.Encapsulation.Type.DEFAULT)]
 
-        label_alloc = label_allocator.LabelAllocator()
+        label_alloc = identifier_allocators.LabelAllocator()
         bgp_manager = mock.Mock()
         bgp_manager.get_local_address.return_value = LOCAL_ADDRESS
-        rd_alloc = rd_allocator.RDAllocator(bgp_manager.get_local_address())
+        rd_alloc = (
+            identifier_allocators.RDAllocator(bgp_manager.get_local_address())
+        )
         self.manager = mock.Mock(bgp_manager=bgp_manager,
                                  label_allocator=label_alloc,
                                  rd_allocator=rd_alloc)
@@ -938,14 +967,14 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
         self.manager.redirect_traffic_to_vpn.reset_mock()
         self.manager.stop_redirect_to_vpn.reset_mock()
 
-    def _generate_route_nlri(self, ip_address_prefix):
+    def _generate_route_nlri(self, ip_address_prefix, nexthop=NEXT_HOP):
         # Parse address/mask
         (_, prefix_len) = self.vpn._parse_ipaddress_prefix(ip_address_prefix)
 
         prefix_rd = self.manager.rd_allocator.get_new_rd(
             "Route distinguisher for prefix %s" % ip_address_prefix
         )
-        rd = self.vpn.instance_rd if prefix_len == 32 else prefix_rd
+        rd = prefix_rd if prefix_len == 32 else self.vpn.instance_rd
 
         label = self.manager.label_allocator.get_new_label(
             "Label for prefix %s" % ip_address_prefix
@@ -953,17 +982,33 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
 
         return ipvpn_routes.IPVPNRouteFactory(exa.AFI(exa.AFI.ipv4),
                                               ip_address_prefix,
-                                              label, rd, NEXT_HOP)
+                                              label, rd, nexthop)
 
     def _generate_flow_spec_nlri(self, classifier):
-        flow_nlri = flowspec.FlowRouteFactory(exa.AFI(exa.AFI.ipv4),
-                                              self.vpn.instance_rd)
+        rd = self.manager.rd_allocator.get_new_rd(
+            "Route distinguisher for FlowSpec NLRI"
+        )
+        flow_nlri = flowspec.FlowRouteFactory(exa.AFI(exa.AFI.ipv4), rd)
 
         for rule in classifier.map_traffic_classifier_2_redirect_rules():
             flow_nlri.add(rule)
 
         return flow_nlri
 
+    def test_validate_convert_attach(self):
+        params = api_params()
+        params.pop('ip_address')
+        self.assertRaises(exc.APIMissingParameterException,
+                          ipvpn.VRF.validate_convert_attach_params,
+                          params)
+
+    def test_validate_convert_detach(self):
+        params = api_params()
+        params.pop('ip_address')
+        self.assertRaises(exc.APIMissingParameterException,
+                          ipvpn.VRF.validate_convert_detach_params,
+                          params)
+
     # unit test for IPVPN re-advertisement
     def test_re_advertisement_1(self):
         self._reset_mocks()
@@ -998,6 +1043,10 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
                                                           '_advertise_route'))
         self.assertIn(RTRecord1, _extract_rtrec_from_call(self.vpn,
                                                           '_advertise_route'))
+        # check that event is for re-advertised route vpn_nlri_2 and
+        #  contains what we expect
+        route_entry = self.vpn._advertise_route.call_args_list[0][0][0]
+        self.assertNotEqual(vpn_nlri_2.rd, route_entry.nlri.rd)
         # dataplane *not* supposed to be updated for this route
         self.mock_dp.setup_dataplane_for_remote_endpoint.assert_not_called()
 
@@ -1024,10 +1073,13 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
         # check that second event is for re-advertised route vpn_nlri_2 and
         #  contains what we expect
         route_entry = self.vpn._advertise_route.call_args_list[1][0][0]
+        vpn_nlri_2_readv_rd = route_entry.nlri.rd
         self.assertEqual(vpn_nlri_2.cidr.prefix(),
                          route_entry.nlri.cidr.prefix())
         self.assertNotEqual(vpn_nlri_2.labels, route_entry.nlri.labels)
         self.assertNotEqual(vpn_nlri_2.nexthop, route_entry.nlri.nexthop)
+        self.assertNotEqual(vpn_nlri_2.rd, route_entry.nlri.rd)
+        self.assertEqual(vpn_nlri_2_readv_rd, route_entry.nlri.rd)
 
         self._reset_mocks()
 
@@ -1052,6 +1104,9 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
                          route_entry.nlri.cidr.prefix())
         self.assertNotEqual(vpn_nlri_2.labels, route_entry.nlri.labels)
         self.assertNotEqual(vpn_nlri_2.nexthop, route_entry.nlri.nexthop)
+        self.assertNotEqual(vpn_nlri_2.rd, route_entry.nlri.rd)
+        self.assertNotEqual(vpn_nlri_2.rd, route_entry.nlri.rd)
+        self.assertEqual(vpn_nlri_2_readv_rd, route_entry.nlri.rd)
 
         self._reset_mocks()
 
@@ -1074,6 +1129,98 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
         # dataplane *not* supposed to be updated for this route
         self.mock_dp.setup_dataplane_for_remote_endpoint.assert_not_called()
 
+    def test_re_advertisement_last(self):
+        self._reset_mocks()
+
+        self.vpn.vif_plugged(MAC1, IP1, LOCAL_PORT1)
+
+        worker_a = worker.Worker(mock.Mock(), 'worker.Worker-A')
+
+        vpn_nlri_1 = self._generate_route_nlri(IP_ADDR_PREFIX1, t.NH1)
+        event1 = self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri_1,
+                                       [t.RT3], worker_a, t.NH1, 200,
+                                       rtrecords=[RTRecord1])
+        # advertised route count increment by 2:
+        # - vif route for port
+        # - re-advertisement of VPN NLRI1 with port as next-hop
+        self.assertEqual(2, self.vpn._advertise_route.call_count)
+        self.vpn._withdraw_route.assert_not_called()
+
+        self._reset_mocks()
+
+        vpn_nlri_1bis = self._generate_route_nlri(IP_ADDR_PREFIX1, t.NH1)
+        event1bis = self._new_route_event(engine.RouteEvent.ADVERTISE,
+                                          vpn_nlri_1bis, [t.RT3], worker_a,
+                                          t.NH1, 200, rtrecords=[RTRecord1])
+        # second re-advertisement of VPN NLRI1 supposed to happen
+        # (must be futher fixed to only append once)
+        self.vpn._advertise_route.assert_called_once()
+        self.vpn._withdraw_route.assert_not_called()
+
+        self._reset_mocks()
+
+        self._revert_event(event1bis)
+        self.vpn._advertise_route.assert_not_called()
+        self.vpn._withdraw_route.assert_not_called()
+
+        self._reset_mocks()
+
+        self._revert_event(event1)
+        # withdraw of re-adv route supposed to happen
+        self.vpn._withdraw_route.assert_called_once()
+        self.vpn._advertise_route.assert_not_called()
+
+    # unit test for FlowSpec re-advertisement
+    def test_flowspec_re_advertisement_1(self):
+        # Configure VRF to generate traffic redirection, based on a 5-tuple
+        # classifier, to a specific route target
+        self._config_vrf_with_attract_traffic(ATTRACT_TRAFFIC_1)
+
+        self.vpn.vif_plugged(MAC1, IP1, LOCAL_PORT1)
+
+        self._reset_mocks()
+
+        worker_a = worker.Worker(mock.Mock(), 'worker.Worker-A')
+
+        # FlowSpec route
+        flow_nlri1 = self._generate_flow_spec_nlri(TC1)
+        self._new_flow_event(engine.RouteEvent.ADVERTISE, flow_nlri1, [t.RT5],
+                             [t.RT3], worker_a)
+
+        # re-advertisement of Flow NLRI1 supposed to happen, to RT4
+        self.assertEqual(2, self.vpn._advertise_route.call_count)
+        self.manager.redirect_traffic_to_vpn.assert_not_called()
+
+        # 1 - re-advertisement of a default route supposed to happen
+        # to RT4
+        self.assertIn(t.RT4,
+                      _extract_rt_from_call(self.vpn, '_advertise_route', 0))
+
+        ipvpn_nlri = _extract_nlri_from_call(self.vpn, '_advertise_route', 0)
+        self.assertEqual(ipvpn.DEFAULT_ADDR_PREFIX, ipvpn_nlri.cidr.prefix())
+
+        # 2 - advertisement of FlowSpec NLRI supposed to happen to RT4
+        #     for traffic redirection to RT5 on TCP destination port 80
+        self.assertIn(t.RT4,
+                      _extract_rt_from_call(self.vpn, '_advertise_route', 1))
+        self.assertNotIn(t.RT3,
+                         _extract_rt_from_call(self.vpn,
+                                               '_advertise_route', 1))
+        self.assertIn(RTRecord3,
+                      _extract_rtrec_from_call(self.vpn,
+                                               '_advertise_route', 1))
+        self.assertEqual(
+            t.RT5,
+            _extract_traffic_redirect_from_call(self.vpn,
+                                                '_advertise_route', 1))
+        # check that second event is for re-advertised route flow_nlri1 and
+        #  contains what we expect
+        route_entry = self.vpn._advertise_route.call_args_list[1][0][0]
+        self.assertNotEqual(flow_nlri1.rd, route_entry.nlri.rd)
+        self.assertEqual(self.vpn.instance_rd, route_entry.nlri.rd)
+        # dataplane *not* supposed to be updated for this route
+        self.mock_dp.setup_dataplane_for_remote_endpoint.assert_not_called()
+
     def _check_attract_traffic(self, method, redirect_rts,
                                expected_classifiers):
         self.assertEqual(len(expected_classifiers),
@@ -1094,7 +1241,8 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
                               _extract_rt_from_call(self.vpn, method, index))
 
                 ipvpn_nlri = _extract_nlri_from_call(self.vpn, method, index)
-                self.assertEqual(DEFAULT_ADDR_PREFIX, ipvpn_nlri.cidr.prefix())
+                self.assertEqual(ipvpn.DEFAULT_ADDR_PREFIX,
+                                 ipvpn_nlri.cidr.prefix())
 
                 if self.vpn.readvertise:
                     self.assertNotIn(self.vpn.readvertise_from_rts[0],
@@ -1106,6 +1254,7 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
                 flow_nlri = _extract_nlri_from_call(self.vpn, method, index)
                 self.assertIsInstance(flow_nlri, exa.Flow)
 
+                self.assertEqual(flow_nlri.rd, self.vpn.instance_rd)
                 self.assertIn(redirect_rts[0],
                               _extract_rt_from_call(self.vpn, method, index))
                 self.assertEqual(
@@ -1120,6 +1269,76 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
                 )
 
     # unit test for IPVPN traffic redirection
+    def test_attract_traffic_re_advertisement(self):
+        # Configure VRF to generate traffic redirection, based on a 5-tuple
+        # classifier, to a specific route target
+        self._config_vrf_with_attract_traffic(ATTRACT_TRAFFIC_1)
+
+        self.vpn.vif_plugged(MAC1, IP1, LOCAL_PORT1)
+
+        # new Route for plugged if supposed to be advertised
+        self.vpn._advertise_route.assert_called_once()
+
+        self._reset_mocks()
+
+        worker_a = worker.Worker(mock.Mock(), 'worker.Worker-A')
+
+        vpn_nlri1 = self._generate_route_nlri(IP_ADDR_PREFIX1)
+        event1 = self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri1,
+                                       [t.RT3], worker_a, t.NH1, 200)
+
+        # Default and FlowSpec routes are supposed to be advertised
+        self.assertEqual(2, self.vpn._advertise_route.call_count)
+        self.vpn._withdraw_route.assert_not_called()
+
+        ipvpn_nlri = _extract_nlri_from_call(self.vpn, '_advertise_route', 0)
+        self.assertIsInstance(ipvpn_nlri, exa.IPVPN)
+        self.assertEqual(ipvpn.DEFAULT_ADDR_PREFIX,
+                         ipvpn_nlri.cidr.prefix())
+
+        flow_nlri = _extract_nlri_from_call(self.vpn, '_advertise_route', 1)
+        self.assertIsInstance(flow_nlri, exa.Flow)
+
+        self._reset_mocks()
+
+        vpn_nlri2 = self._generate_route_nlri(IP_ADDR_PREFIX2)
+        event2 = self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri2,
+                                       [t.RT3], worker_a, t.NH1, 200)
+
+        # Only FlowSpec route is supposed to be advertised
+        self.vpn._advertise_route.assert_called_once()
+        self.vpn._withdraw_route.assert_not_called()
+
+        flow_nlri = _extract_nlri_from_call(self.vpn, '_advertise_route', 0)
+        self.assertIsInstance(flow_nlri, exa.Flow)
+
+        self._reset_mocks()
+
+        self._revert_event(event2)
+
+        # Only FlowSpec route is supposed to be withdrawn
+        self.vpn._withdraw_route.assert_called_once()
+        self.vpn._advertise_route.assert_not_called()
+
+        flow_nlri = _extract_nlri_from_call(self.vpn, '_withdraw_route', 0)
+        self.assertIsInstance(flow_nlri, exa.Flow)
+
+        self._reset_mocks()
+
+        self._revert_event(event1)
+
+        # Default and FlowSpec routes are supposed to be withdrawn
+        self.assertEqual(2, self.vpn._withdraw_route.call_count)
+        self.vpn._advertise_route.assert_not_called()
+
+        ipvpn_nlri = _extract_nlri_from_call(self.vpn, '_withdraw_route', 0)
+        self.assertIsInstance(ipvpn_nlri, exa.IPVPN)
+        self.assertEqual(ipvpn.DEFAULT_ADDR_PREFIX,
+                         ipvpn_nlri.cidr.prefix())
+
+        flow_nlri = _extract_nlri_from_call(self.vpn, '_withdraw_route', 1)
+        self.assertIsInstance(flow_nlri, exa.Flow)
+
     def test_attract_traffic_single_prefix_advertise(self):
         # Configure VRF to generate traffic redirection, based on a 5-tuple
         # classifier, to a specific route target
@@ -1157,15 +1376,14 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
         worker_a = worker.Worker(mock.Mock(), 'worker.Worker-A')
 
         vpn_nlri1 = self._generate_route_nlri(IP_ADDR_PREFIX1)
-        self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri1, [t.RT3],
-                              worker_a, t.NH1, 200)
+        event1 = self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri1,
+                                       [t.RT3], worker_a, t.NH1, 200)
 
         self.assertEqual(2, self.vpn._advertise_route.call_count)
 
         self._reset_mocks()
 
-        self._new_route_event(engine.RouteEvent.WITHDRAW, vpn_nlri1, [t.RT3],
-                              worker_a, t.NH1, 200)
+        self._revert_event(event1)
 
         self._check_attract_traffic('_withdraw_route',
                                     ATTRACT_TRAFFIC_1['redirect_rts'],
@@ -1196,7 +1414,7 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
         self._check_attract_traffic(
             '_advertise_route',
             ATTRACT_TRAFFIC_1['redirect_rts'],
-            [None, TC1, None, TC2])
+            [None, TC1, TC2])
 
     def test_attract_traffic_multiple_prefix_withdraw(self):
         # Configure VRF to generate traffic redirection, based on a 5-tuple
@@ -1213,27 +1431,24 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
         worker_a = worker.Worker(mock.Mock(), 'worker.Worker-A')
 
         vpn_nlri1 = self._generate_route_nlri(IP_ADDR_PREFIX1)
-        self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri1, [t.RT3],
-                              worker_a, t.NH1, 200)
+        event1 = self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri1,
+                                       [t.RT3], worker_a, t.NH1, 200)
 
         vpn_nlri2 = self._generate_route_nlri(IP_ADDR_PREFIX2)
-        self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri2, [t.RT3],
-                              worker_a, t.NH1, 200)
+        event2 = self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri2,
+                                       [t.RT3], worker_a, t.NH1, 200)
 
-        self.assertEqual(4, self.vpn._advertise_route.call_count)
+        self.assertEqual(3, self.vpn._advertise_route.call_count)
 
         self._reset_mocks()
 
-        self._new_route_event(engine.RouteEvent.WITHDRAW, vpn_nlri2, [t.RT3],
-                              worker_a, t.NH1, 200)
-
-        self._new_route_event(engine.RouteEvent.WITHDRAW, vpn_nlri1, [t.RT3],
-                              worker_a, t.NH1, 200)
+        self._revert_event(event2)
+        self._revert_event(event1)
 
         self._check_attract_traffic(
             '_withdraw_route',
             ATTRACT_TRAFFIC_1['redirect_rts'],
-            [None, TC2, None, TC1])
+            [TC2, None, TC1])
 
     def test_attract_traffic_static_dest_prefix_no_readvertise_advertise(self):
         # Configure VRF to generate traffic redirection, based on a 5-tuple
@@ -1370,25 +1585,23 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
         # FlowSpec route
 
         flow_nlri1 = self._generate_flow_spec_nlri(TC1)
-        self._new_flow_event(engine.RouteEvent.ADVERTISE, flow_nlri1,
-                             [t.RT5], [t.RT1], worker_a)
+        event1 = self._new_flow_event(engine.RouteEvent.ADVERTISE, flow_nlri1,
+                                      [t.RT5], [t.RT1], worker_a)
         flow_nlri2 = self._generate_flow_spec_nlri(TC2)
-        self._new_flow_event(engine.RouteEvent.ADVERTISE, flow_nlri2,
-                             [t.RT5], [t.RT1], worker_a)
+        event2 = self._new_flow_event(engine.RouteEvent.ADVERTISE, flow_nlri2,
+                                      [t.RT5], [t.RT1], worker_a)
 
         self.assertEqual(2, self.manager.redirect_traffic_to_vpn.call_count)
 
         self._reset_mocks_vpnmanager()
 
-        self._new_flow_event(engine.RouteEvent.WITHDRAW, flow_nlri2, [t.RT5],
-                             [t.RT1], worker_a)
+        self._revert_event(event2)
 
         redirect_rt5 = t._rt_to_string(t.RT5)
         self.assertNotIn(TC2,
                          self.vpn.redirect_rt_2_classifiers[redirect_rt5])
 
-        self._new_flow_event(engine.RouteEvent.WITHDRAW, flow_nlri1, [t.RT5],
-                             [t.RT1], worker_a)
+        self._revert_event(event1)
 
         self.assertTrue(not self.vpn.redirect_rt_2_classifiers)
         self.manager.stop_redirect_to_vpn.assert_called_once()
@@ -1432,15 +1645,14 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
         worker_a = worker.Worker(mock.Mock(), 'worker.Worker-A')
 
         vpn_nlri1 = self._generate_route_nlri(IP_ADDR_PREFIX1)
-        self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri1, [t.RT3],
-                              worker_a, t.NH1, 200)
+        event1 = self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri1,
+                                       [t.RT3], worker_a, t.NH1, 200)
 
         self.assertEqual(3, self.vpn._advertise_route.call_count)
 
         self._reset_mocks()
 
-        self._new_route_event(engine.RouteEvent.WITHDRAW, vpn_nlri1, [t.RT3],
-                              worker_a, t.NH1, 200)
+        self._revert_event(event1)
 
         self._check_attract_traffic('_withdraw_route',
                                     ATTRACT_TRAFFIC_1['redirect_rts'],
@@ -1472,7 +1684,7 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
         self._check_attract_traffic(
             '_advertise_route',
             ATTRACT_TRAFFIC_1['redirect_rts'],
-            [None, None, TC1, None, None, TC2])
+            [None, None, TC1, TC2])
 
     def test_load_balancing_multiple_prefix_withdraw(self):
         # Configure VRF to generate traffic redirection, based on a 5-tuple
@@ -1490,26 +1702,24 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
         worker_a = worker.Worker(mock.Mock(), 'worker.Worker-A')
 
         vpn_nlri1 = self._generate_route_nlri(IP_ADDR_PREFIX1)
-        self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri1, [t.RT3],
-                              worker_a, t.NH1, 200)
+        event1 = self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri1,
+                                       [t.RT3], worker_a, t.NH1, 200)
 
         vpn_nlri2 = self._generate_route_nlri(IP_ADDR_PREFIX2)
-        self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri2, [t.RT3],
-                              worker_a, t.NH1, 200)
+        event2 = self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri2,
+                                       [t.RT3], worker_a, t.NH1, 200)
 
-        self.assertEqual(6, self.vpn._advertise_route.call_count)
+        self.assertEqual(4, self.vpn._advertise_route.call_count)
 
         self._reset_mocks()
 
-        self._new_route_event(engine.RouteEvent.WITHDRAW, vpn_nlri2, [t.RT3],
-                              worker_a, t.NH1, 200)
-        self._new_route_event(engine.RouteEvent.WITHDRAW, vpn_nlri1, [t.RT3],
-                              worker_a, t.NH1, 200)
+        self._revert_event(event2)
+        self._revert_event(event1)
 
         self._check_attract_traffic(
             '_withdraw_route',
             ATTRACT_TRAFFIC_1['redirect_rts'],
-            [None, None, TC2, None, None, TC1])
+            [TC2, None, None, TC1])
 
     def test_load_balancing_new_plug(self):
         # Configure VRF to generate traffic redirection, based on a 5-tuple
@@ -1539,7 +1749,7 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
         self._check_attract_traffic(
             '_advertise_route',
             ATTRACT_TRAFFIC_1['redirect_rts'],
-            [None, None, TC1])
+            [None, None])
 
     def test_load_balancing_unplug_all(self):
         # Configure VRF to generate traffic redirection, based on a 5-tuple
@@ -1579,3 +1789,27 @@ class TestVRF(t.BaseTestBagPipeBGP, testtools.TestCase):
             '_withdraw_route',
             ATTRACT_TRAFFIC_1['redirect_rts'],
             [None, TC1, None])
+
+    def test_cleanup_assist(self):
+        # Configure VRF to generate traffic redirection, based on a 5-tuple
+        # classifier, to a specific route target
+        self._config_vrf_with_attract_traffic(ATTRACT_TRAFFIC_1)
+
+        worker_a = worker.Worker(mock.Mock(), 'worker.Worker-A')
+
+        vpn_nlri1 = self._generate_route_nlri(IP_ADDR_PREFIX1)
+        self._new_route_event(engine.RouteEvent.ADVERTISE, vpn_nlri1,
+                              [t.RT1], worker_a, t.NH1, 200)
+
+        # FlowSpec route
+        flow_nlri1 = self._generate_flow_spec_nlri(TC1)
+        self._new_flow_event(engine.RouteEvent.ADVERTISE, flow_nlri1,
+                             [t.RT5], [t.RT1], worker_a)
+
+        self.mock_dp.needs_cleanup_assist.return_value = False
+
+        with mock.patch.object(self.vpn, 'best_route_removed') as mock_brr:
+            self.vpn.stop()
+            mock_brr.assert_called_once()
+            self.assertIsInstance(mock_brr.call_args[0][1].nlri,
+                                  flowspec.Flow)
