@@ -1,6 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-# encoding: utf-8
-
 # Copyright 2014 Orange
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,11 +12,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import mock
-
-from networking_bagpipe.bagpipe_bgp import constants as consts
+from unittest import mock
 
+from networking_bagpipe.bagpipe_bgp.common import exceptions
 from networking_bagpipe.bagpipe_bgp.common import utils
+from networking_bagpipe.bagpipe_bgp import constants as consts
 from networking_bagpipe.bagpipe_bgp.vpn import manager
 from networking_bagpipe.tests.unit.bagpipe_bgp import base as t
 
@@ -31,7 +28,8 @@ MAC = "00:00:de:ad:be:ef"
 IP = "10.0.0.1/32"
 BRIDGE_NAME = "br-test"
 LOCAL_PORT = {'linuxif': 'tap1'}
-VPN_EXT_ID = 1
+VPN_EXT_ID = "ext_id_1"
+VPN_EXT_ID_bis = "ext_id_2"
 GW_IP = "10.0.0.1"
 GW_MASK = 24
 VNID = 255
@@ -54,6 +52,8 @@ class MockVPNInstance(object):
         self.import_rts = import_rts
         self.export_rts = export_rts
 
+        self.forced_vni = False
+
     def __repr__(self):
         return "%s:%s:%s" % (self.instance_type,
                              self.instance_id,
@@ -70,6 +70,9 @@ class MockVPNInstance(object):
     def update_fallback(self, *args):
         pass
 
+    def update_route_targets(self, *args):
+        pass
+
     def vif_plugged(self, *args, **kwargs):
         pass
 
@@ -159,7 +162,9 @@ class TestVPNManager(t.TestCase):
         self.assertTrue(not self.manager.vpn_instances)
 
     def test_plug_vif_to_vpn_with_forced_vni(self):
-        with mock.patch.object(self.manager, "_get_vpn_instance") as mock_get_vpn_instance, \
+        with mock.patch.object(self.manager, "_get_vpn_instance",
+                               return_value=(mock.Mock(), False)
+                               ) as mock_get_vpn_instance, \
                 mock.patch.object(utils, "convert_route_targets"):
             self.manager.plug_vif_to_vpn(vpn_instance_id=VPN_EXT_ID,
                                          vpn_type=consts.EVPN,
@@ -176,7 +181,9 @@ class TestVPNManager(t.TestCase):
             None, None, None, linuxbr=BRIDGE_NAME, vni=VNID)
 
     def test_plug_vif_to_vpn_without_forced_vni(self):
-        with mock.patch.object(self.manager, "_get_vpn_instance") as mock_get_vpn_instance, \
+        with mock.patch.object(self.manager, "_get_vpn_instance",
+                               return_value=(mock.Mock(), False)
+                               ) as mock_get_vpn_instance, \
                 mock.patch.object(utils, "convert_route_targets"):
             self.manager.plug_vif_to_vpn(vpn_instance_id=VPN_EXT_ID,
                                          vpn_type=consts.EVPN,
@@ -193,26 +200,59 @@ class TestVPNManager(t.TestCase):
             None, None, None, linuxbr=BRIDGE_NAME)
 
     def test_get_vpn_instance_with_forced_vni(self):
-        vpn_instance = self.manager._get_vpn_instance(VPN_EXT_ID,
+        instannce, _ = self.manager._get_vpn_instance(VPN_EXT_ID,
                                                       consts.IPVPN,
                                                       [], [],
                                                       GW_IP, GW_MASK,
                                                       None, None,
                                                       vni=VNID)
+        instannce.start()
 
-        self.assertEqual(VNID, vpn_instance.instance_label,
+        self.assertEqual(VNID, instannce.instance_label,
                          "VPN instance label should be forced to VNID")
 
     def test_get_vpn_instance_without_forced_vni(self):
-        vpn_instance = self.manager._get_vpn_instance(VPN_EXT_ID,
+        instannce, _ = self.manager._get_vpn_instance(VPN_EXT_ID,
                                                       consts.IPVPN,
                                                       [], [],
                                                       GW_IP, GW_MASK,
                                                       None, None)
 
-        self.assertIsNot(0, vpn_instance.instance_label,
+        instannce.start()
+
+        self.assertIsNot(0, instannce.instance_label,
                          "VPN instance label should be assigned locally")
 
+    def test_forced_vni_same_vni_twice(self):
+        instannce, _ = self.manager._get_vpn_instance(VPN_EXT_ID,
+                                                      consts.IPVPN,
+                                                      [], [],
+                                                      GW_IP, GW_MASK,
+                                                      None, None,
+                                                      vni=VNID)
+        instannce.start()
+
+        self.assertRaises(exceptions.APIAlreadyUsedVNI,
+                          self.manager._get_vpn_instance,
+                          VPN_EXT_ID_bis,
+                          consts.EVPN,
+                          [], [],
+                          GW_IP, GW_MASK,
+                          None, None,
+                          vni=VNID)
+
+        # unregister first VPN instance (free the VNI)
+        self.manager.unregister_vpn_instance(instannce)
+
+        # this time, using the VNI should work
+        instance2, _ = self.manager._get_vpn_instance(VPN_EXT_ID,
+                                                      consts.IPVPN,
+                                                      [], [],
+                                                      GW_IP, GW_MASK,
+                                                      None, None,
+                                                      vni=VNID)
+        instance2.start()
+
     def test_instance_id_uniqueness(self):
         with mock.patch.object(manager.VPNManager, 'type2class',
                                {consts.IPVPN: MockVPNInstance,
@@ -232,17 +272,17 @@ class TestVPNManager(t.TestCase):
 
             for i in (1, 2, 3, 4, 5):
                 self.manager.plug_vif_to_vpn(
-                    external_instance_id=BASE_VPN_EXT+str(i),
+                    external_instance_id=BASE_VPN_EXT + str(i),
                     **vpn_instance_plug_args)
 
             for i in (2, 4):
                 self.manager.unplug_vif_from_vpn(
-                    external_instance_id=BASE_VPN_EXT+str(i),
+                    external_instance_id=BASE_VPN_EXT + str(i),
                     **vpn_instance_unplug_args)
 
             for i in (6, 7, 8):
                 self.manager.plug_vif_to_vpn(
-                    external_instance_id=BASE_VPN_EXT+str(i),
+                    external_instance_id=BASE_VPN_EXT + str(i),
                     **vpn_instance_plug_args)
 
             instance_ids = [i.instance_id
@@ -281,8 +321,10 @@ class TestVPNManager(t.TestCase):
 
     @mock.patch('networking_bagpipe.bagpipe_bgp.engine.bgp_manager.Manager')
     def test_manager_stop(self, mocked_bgp_manager):
-        self.manager._get_vpn_instance("TEST_VPN_INSTANCE", consts.IPVPN,
-                                       [t.RT1], [t.RT1], "192.168.0.1", 24,
-                                       {}, {})
+        instance, _ = self.manager._get_vpn_instance(
+            "TEST_VPN_INSTANCE", consts.IPVPN, [t.RT1], [t.RT1],
+            "192.168.0.1", 24, {}, {})
+        instance.start()
+
         self.manager.stop()
         self.assertTrue(not self.manager.vpn_instances)
