@@ -12,20 +12,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
 import random
+from unittest import mock
 
 import netaddr
 from oslo_utils import uuidutils
 
 from networking_bagpipe.objects import bgpvpn as bgpvpn_obj
 
-from neutron.common import utils
-from neutron.objects import network
-from neutron.objects import ports
-from neutron.objects import router
-from neutron.objects import subnet
-
 from neutron.tests.unit.objects import test_base
 from neutron.tests.unit import testlib_api
 
@@ -33,6 +27,8 @@ from neutron_lib.api.definitions import bgpvpn as bgpvpn_api
 from neutron_lib.api.definitions import bgpvpn_routes_control as bgpvpn_rc_api
 from neutron_lib import constants
 from neutron_lib import context
+from neutron_lib.objects import registry as obj_reg
+from neutron_lib.utils import net as net_utils
 
 
 test_base.FIELD_TYPE_VALUE_GENERATOR_MAP[bgpvpn_obj.BGPVPNTypeField] = (
@@ -56,7 +52,7 @@ def _subnet_dict(gw_mac=None):
         'id': mock.ANY,
         'ip_version': 4,
         'gateway_mac': gw_mac,
-        'cidr': utils.AuthenticIPNetwork(CIDR),
+        'cidr': net_utils.AuthenticIPNetwork(CIDR),
         'gateway_ip': netaddr.IPAddress(GW_IP)
     }
 
@@ -75,32 +71,35 @@ class _BPGVPNObjectsTestCommon(object):
         return self._create_test_bgpvpn().id
 
     def _make_subnet(self, network_id):
-        _subnet = subnet.Subnet(self.context,
-                                network_id=network_id,
-                                ip_version=4,
-                                cidr=netaddr.IPNetwork(CIDR),
-                                gateway_ip=GW_IP)
+        _subnet = obj_reg.new_instance(
+            'Subnet', self.context,
+            network_id=network_id,
+            ip_version=4,
+            cidr=netaddr.IPNetwork(CIDR),
+            gateway_ip=GW_IP)
         _subnet.create()
         return _subnet
 
     def _connect_router_network(self, router_id, network_id,
                                 subnet_id=None, gw_network=False):
-        port = ports.Port(self.context,
-                          network_id=network_id,
-                          mac_address=netaddr.EUI(
-                              GW_MAC,
-                              dialect=netaddr.mac_unix_expanded),
-                          device_id='test_device_id',
-                          device_owner=constants.DEVICE_OWNER_ROUTER_INTF,
-                          status="DUMMY_STATUS",
-                          admin_state_up=True)
+        port = obj_reg.new_instance(
+            'Port', self.context,
+            network_id=network_id,
+            mac_address=netaddr.EUI(
+                GW_MAC,
+                dialect=netaddr.mac_unix_expanded),
+            device_id='test_device_id',
+            device_owner=constants.DEVICE_OWNER_ROUTER_INTF,
+            status="DUMMY_STATUS",
+            admin_state_up=True)
         if gw_network:
             port.device_owner = constants.DEVICE_OWNER_ROUTER_GW
 
         port.create()
 
         if subnet_id:
-            allocation = ports.IPAllocation(
+            allocation = obj_reg.new_instance(
+                'IPAllocation',
                 self.context,
                 port_id=port.id,
                 subnet_id=subnet_id,
@@ -111,9 +110,10 @@ class _BPGVPNObjectsTestCommon(object):
             port.fixed_ips = [allocation]
             port.update()
 
-        router_if = router.RouterPort(self.context,
-                                      router_id=router_id,
-                                      port_id=port.id)
+        router_if = obj_reg.new_instance(
+            'RouterPort', self.context,
+            router_id=router_id,
+            port_id=port.id)
         router_if.create()
 
 
@@ -153,16 +153,16 @@ class BGPVPNNetAssociationTest(test_base.BaseDbObjectTestCase,
 
     def test_all_subnets(self):
         for db_obj in self.objs:
-            self.assertItemsEqual(db_obj.all_subnets(self.network_id),
+            self.assertCountEqual(db_obj.all_subnets(self.network_id),
                                   [_subnet_dict()])
 
     def test_subnets(self):
         for obj in self.objs:
             obj.create()
-            self.assertItemsEqual(obj.subnets, [_subnet_dict()])
+            self.assertCountEqual(obj.subnets, [_subnet_dict()])
 
         # plug a router
-        _router = router.Router(self.context)
+        _router = obj_reg.new_instance('Router', self.context)
         _router.create()
 
         self._connect_router_network(_router.id,
@@ -174,7 +174,7 @@ class BGPVPNNetAssociationTest(test_base.BaseDbObjectTestCase,
             refreshed_obj = bgpvpn_obj.BGPVPNNetAssociation.get_object(
                 self.context,
                 id=obj.id)
-            self.assertItemsEqual(refreshed_obj.subnets,
+            self.assertCountEqual(refreshed_obj.subnets,
                                   [_subnet_dict(GW_MAC)])
 
 
@@ -190,7 +190,7 @@ class BGPVPNRouterAssociationTest(test_base.BaseDbObjectTestCase,
         self.update_obj_fields(
             {'router_id': self.router_id,
              'bgpvpn_id': self._create_test_bgpvpn_id,
-             'routes': []})
+             })
         self.context = context.get_admin_context()
 
     def test_get_objects_queries_constant(self):
@@ -205,7 +205,7 @@ class BGPVPNRouterAssociationTest(test_base.BaseDbObjectTestCase,
 
         # initially the network is not connected to the router
         for obj in self.objs:
-            self.assertItemsEqual(obj.all_subnets(network_id), [])
+            self.assertCountEqual(obj.all_subnets(network_id), [])
 
         self._connect_router_network(self.router_id,
                                      network_id,
@@ -223,19 +223,19 @@ class BGPVPNRouterAssociationTest(test_base.BaseDbObjectTestCase,
             refreshed_obj = bgpvpn_obj.BGPVPNRouterAssociation.get_object(
                 self.context,
                 id=obj.id)
-            self.assertItemsEqual(refreshed_obj.all_subnets(network_id),
+            self.assertCountEqual(refreshed_obj.all_subnets(network_id),
                                   [_subnet_dict(GW_MAC)])
-            self.assertItemsEqual(refreshed_obj.all_subnets("dummy-uuid"),
+            self.assertCountEqual(refreshed_obj.all_subnets("dummy-uuid"),
                                   [])
 
     def test_get_objects_from_network_id(self):
-        router_ = router.Router(self.context)
+        router_ = obj_reg.new_instance('Router', self.context)
         router_.create()
 
         self.project = uuidutils.generate_uuid()
 
         # put a network behind a router
-        network_ = network.Network(self.context)
+        network_ = obj_reg.new_instance('Network', self.context)
         network_.create()
 
         subnet_ = self._make_subnet(network_.id)
@@ -253,7 +253,7 @@ class BGPVPNRouterAssociationTest(test_base.BaseDbObjectTestCase,
         router_assoc_.create()
 
         # unrelated router and BGPVPN
-        router_2 = router.Router(self.context)
+        router_2 = obj_reg.new_instance('Router', self.context)
         router_2.create()
         router_assoc_2 = bgpvpn_obj.BGPVPNRouterAssociation(
             self.context,
@@ -289,7 +289,8 @@ class BGPVPNPortAssociationTest(test_base.BaseDbObjectTestCase,
              'bgpvpn_id': self._create_test_bgpvpn_id,
              'routes': {
                  'bgpvpn_id': self._create_test_bgpvpn_id,
-                 }})
+             }}
+        )
 
     def test_get_objects_queries_constant(self):
         self.skipTest("test not passing yet, remains to be investigated why")
@@ -329,8 +330,8 @@ class BGPVPNPortAssociationRouteTest(test_base.BaseDbObjectTestCase,
         r2 = bgpvpn_obj.BGPVPNPortAssociationRoute(
             type='prefix',
             prefix=netaddr.IPNetwork('1.2.3.4'))
-        self.assertTrue(r1 == r2)
-        self.assertTrue(hash(r1) == hash(r2))
+        self.assertEqual(r1, r2)
+        self.assertEqual(hash(r1), hash(r2))
 
     def test_eq_hash_bgpvpn(self):
         bgpvpn = self._create_test_bgpvpn()
@@ -338,8 +339,8 @@ class BGPVPNPortAssociationRouteTest(test_base.BaseDbObjectTestCase,
                                                    bgpvpn=bgpvpn)
         r2 = bgpvpn_obj.BGPVPNPortAssociationRoute(type='bgpvpn',
                                                    bgpvpn=bgpvpn)
-        self.assertTrue(r1 == r2)
-        self.assertTrue(hash(r1) == hash(r2))
+        self.assertEqual(r1, r2)
+        self.assertEqual(hash(r1), hash(r2))
 
     def test_neq_type(self):
         r1 = bgpvpn_obj.BGPVPNPortAssociationRoute(
@@ -348,7 +349,7 @@ class BGPVPNPortAssociationRouteTest(test_base.BaseDbObjectTestCase,
         r2 = bgpvpn_obj.BGPVPNPortAssociationRoute(
             type='prefix',
             prefix=netaddr.IPNetwork('1.2.3.4'))
-        self.assertTrue(r1 != r2)
+        self.assertNotEqual(r1, r2)
 
     def test_neq_prefix(self):
         r1 = bgpvpn_obj.BGPVPNPortAssociationRoute(
@@ -357,7 +358,7 @@ class BGPVPNPortAssociationRouteTest(test_base.BaseDbObjectTestCase,
         r2 = bgpvpn_obj.BGPVPNPortAssociationRoute(
             type='prefix',
             prefix=netaddr.IPNetwork('1.2.3.4'))
-        self.assertTrue(r1 != r2)
+        self.assertNotEqual(r1, r2)
 
     def test_neq_bgpvpn(self):
         bgpvpn1 = self._create_test_bgpvpn()
@@ -366,7 +367,7 @@ class BGPVPNPortAssociationRouteTest(test_base.BaseDbObjectTestCase,
                                                    bgpvpn=bgpvpn1)
         r2 = bgpvpn_obj.BGPVPNPortAssociationRoute(type='bgpvpn',
                                                    bgpvpn=bgpvpn2)
-        self.assertTrue(r1 != r2)
+        self.assertNotEqual(r1, r2)
 
     def test_bgpvpn_route_get_object_access_bgpvpn(self):
         route_id = uuidutils.generate_uuid()
@@ -381,6 +382,6 @@ class BGPVPNPortAssociationRouteTest(test_base.BaseDbObjectTestCase,
         route_again = bgpvpn_obj.BGPVPNPortAssociationRoute.get_object(
             self.context,
             id=route_id
-            )
+        )
 
         self.assertEqual([TEST_RT], route_again.bgpvpn.route_targets)
